import java.awt.*;
import java.awt.event.*;


class AlignDialog extends Dialog implements ActionListener {

  CheckboxGroup horizontalAlign;
  Checkbox horizontalNone;
  Checkbox top;
  Checkbox horizontalCenter;
  Checkbox bottom;
  Checkbox horizontalDistribute;
	
  CheckboxGroup verticalAlign;
  Checkbox verticalNone;
  Checkbox left;
  Checkbox verticalCenter;
  Checkbox right;
  Checkbox verticalDistribute;

  OkCancelPanel ocp;
	

  public static final byte NONE       = 0;
  public static final byte TOP        = 1;
  public static final byte CENTER     = 2;
  public static final byte BOTTOM     = 3;
  public static final byte DISTRIBUTE = 4;
  public static final byte LEFT       = 5;
  public static final byte RIGHT      = 6;
	

  public AlignDialog(Frame f) {
		
    super(f, "Alignment", true);
    setLocation(60, 60);
		
    addWindowListener((WindowListener) new WindowWindowListener(this));

    GridBagLayout gridbag = new GridBagLayout();
    Panel l = new Panel(gridbag);
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.anchor = GridBagConstraints.WEST;
        
    Label horizontalAxis = new Label("Horizontal axis");
    gridbag.setConstraints(horizontalAxis, gbc);
    l.add(horizontalAxis);

    horizontalAlign = new CheckboxGroup();

    horizontalNone = new Checkbox(" None", horizontalAlign, true);
    gridbag.setConstraints(horizontalNone, gbc);
    l.add(horizontalNone);

    top = new Checkbox(" Top", horizontalAlign, false);
    gridbag.setConstraints(top, gbc);
    l.add(top);

    horizontalCenter = new Checkbox(" Center", horizontalAlign, false);
    gridbag.setConstraints(horizontalCenter, gbc);
    l.add(horizontalCenter);

    bottom = new Checkbox(" Bottom", horizontalAlign, false);
    gridbag.setConstraints(bottom, gbc);
    l.add(bottom);

    horizontalDistribute = new Checkbox(" Distribute", horizontalAlign, false);
    gridbag.setConstraints(horizontalDistribute, gbc);
    l.add(horizontalDistribute);

    Panel r = new Panel(gridbag);

    Label verticalAxis = new Label("Vertical axis");
    gridbag.setConstraints(verticalAxis, gbc);
    r.add(verticalAxis);
    
    verticalAlign = new CheckboxGroup();
    
    verticalNone = new Checkbox(" None", verticalAlign, true);
    gridbag.setConstraints(verticalNone, gbc);
    r.add(verticalNone);

    left = new Checkbox(" Left", verticalAlign, false);
    gridbag.setConstraints(left, gbc);
    r.add(left);

    verticalCenter = new Checkbox(" Center", verticalAlign, false);
    gridbag.setConstraints(verticalCenter, gbc);
    r.add(verticalCenter);

    right = new Checkbox(" Right", verticalAlign, false);
    gridbag.setConstraints(right, gbc);
    r.add(right);

    verticalDistribute = new Checkbox(" Distribute", verticalAlign, false);
    gridbag.setConstraints(verticalDistribute, gbc);
    r.add(verticalDistribute);

    Panel t = new InsetsPanel(new Insets(0, 0, 10, 0),
			      new GridLayout(1, 2, 10, 10));
    t.add(l);
    t.add(r);

    ocp = new OkCancelPanel();
    ocp.ok.addActionListener(this);
    ocp.cancel.addActionListener(this);
		
    InsetsPanel mainPanel = new InsetsPanel(new Insets(10, 10, 10, 10),
					    new BorderLayout());
    mainPanel.add("North", t);
    mainPanel.add("South", ocp);

    add(mainPanel);

    pack();
  }


  public void actionPerformed(ActionEvent e) {
    
    if (e.getSource() == ocp.ok) {
      setVisible(false);
      dispose();

      Checkbox h = horizontalAlign.getSelectedCheckbox();
      byte hAxis = NONE;
			
      if (h == top) {
	hAxis = TOP;
      }
      else if (h == horizontalCenter) {
	hAxis = CENTER;
      }
      else if (h == bottom) {
	hAxis = BOTTOM;
      }
      else if (h == horizontalDistribute) {
	hAxis = DISTRIBUTE;
      }

      Checkbox v = verticalAlign.getSelectedCheckbox();
      byte vAxis = NONE;
			
      if (v == left) {
	vAxis = LEFT;
      }
      else if (v == verticalCenter) {
	vAxis = CENTER;
      }
      else if (v == right) {
	vAxis = RIGHT;
      }
      else if (v == verticalDistribute) {
	vAxis = DISTRIBUTE;
      }

      Victor vic = (Victor) getParent();
      if (vic.s.align(hAxis, vAxis)) {
	vic.s.repaint();
      }
    }
    else if (e.getSource() == ocp.cancel) {
      setVisible(false);
      dispose();
    }
  }

}
