import java.awt.*;
import java.awt.event.*;


class ColorSelector extends Dialog implements ActionListener, MouseListener {

  TextField redValue, greenValue, blueValue;
  Canvas selectedColor;
  OkCancelPanel ocp;
	

  public ColorSelector(Frame f) {
		
    super(f, "Color Selection", true);
    setLocation(60, 60);
    addWindowListener((WindowListener) new WindowWindowListener(this));

    Panel systemColors = new InsetsPanel(new Insets(0, 0, 10, 0),
					 new GridLayout(3, 6, 5, 5));

    addSystemColor(systemColors, 255, 0, 0);
    addSystemColor(systemColors, 0, 255, 0);
    addSystemColor(systemColors, 0, 0, 255);
    addSystemColor(systemColors, 0, 255, 255);
    addSystemColor(systemColors, 255, 0, 255);
    addSystemColor(systemColors, 255, 255, 0);

    addSystemColor(systemColors, 128, 0, 0);
    addSystemColor(systemColors, 0, 128, 0);
    addSystemColor(systemColors, 0, 0, 128);
    addSystemColor(systemColors, 0, 128, 128);
    addSystemColor(systemColors, 128, 0, 128);
    addSystemColor(systemColors, 128, 128, 0);
    
    addSystemColor(systemColors, 255, 255, 255);
    addSystemColor(systemColors, 192, 192, 192);
    addSystemColor(systemColors, 160, 160, 164);
    addSystemColor(systemColors, 128, 128, 128);
    addSystemColor(systemColors, 0, 0, 0);
    addSystemColor(systemColors, 192, 192, 192);
    
    GridBagLayout gridbag = new GridBagLayout();
    Panel p = new InsetsPanel(new Insets(0, 0, 10, 0), gridbag);
    GridBagConstraints gbc = new GridBagConstraints();    

    Victor v = (Victor) f;

    gbc.gridx = 0;
    gbc.anchor = GridBagConstraints.WEST;
    
    Label redLabel = new Label("Red");
    gridbag.setConstraints(redLabel, gbc);
    p.add(redLabel);
    
    Label greenLabel = new Label("Green");
    gridbag.setConstraints(greenLabel, gbc);
    p.add(greenLabel);
    
    Label blueLabel = new Label("Blue");
    gridbag.setConstraints(blueLabel, gbc);
    p.add(blueLabel);

    gbc.gridx = 1;
    
    redValue = new TextField(String.valueOf(v.tp.currentColor.getRed()), 3);
    redValue.addActionListener(this);
    gridbag.setConstraints(redValue, gbc);
    p.add(redValue);
    
    greenValue = new TextField(String.valueOf(v.tp.currentColor.getGreen()),
			       3);
    greenValue.addActionListener(this);
    gridbag.setConstraints(greenValue, gbc);
    p.add(greenValue);
    
    blueValue = new TextField(String.valueOf(v.tp.currentColor.getBlue()), 3);
    blueValue.addActionListener(this);
    gridbag.setConstraints(blueValue, gbc);
    p.add(blueValue);

    gbc.gridx = 2;
    gbc.gridheight = 3;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.insets = new Insets(0, 5, 0, 0);
    selectedColor = new GraphicButton(GraphicButton.empty, true);
    selectedColor.setSize(45, 50);
    selectedColor.setBackground(v.tp.currentColor);
    gridbag.setConstraints(selectedColor, gbc);
    p.add(selectedColor);
    
    ocp = new OkCancelPanel();
    ocp.ok.addActionListener(this);
    ocp.cancel.addActionListener(this);

    InsetsPanel mainPanel = new InsetsPanel(new Insets(10, 10, 10, 10),
					    new BorderLayout());
    mainPanel.add("North", systemColors);
    mainPanel.add("Center", p);
    mainPanel.add("South", ocp);

    add(mainPanel);

    pack();
    
  }


  private void addSystemColor(Panel p, int r, int g, int b) {

    GraphicButton c = new GraphicButton(GraphicButton.empty, true);
    c.setSize(20, 20);
    c.setBackground(new Color(r, g, b));
    c.addMouseListener(this);
    p.add(c);

  }

  
  private Color getColor() {

    int r, g, b;
    
    try {
      r = Integer.parseInt(redValue.getText());
      g = Integer.parseInt(greenValue.getText());
      b = Integer.parseInt(blueValue.getText());
    }
    catch (NumberFormatException nfe) {
      return null;
    }

    Color c;
    
    try {
      c = new Color(r, g, b);
    }
    catch (IllegalArgumentException iae) {
      return null;
    }
    
    return c;
    
  }

  
  public void actionPerformed(ActionEvent e) {

    if ((e.getSource() == redValue) ||
	(e.getSource() == greenValue) ||
	(e.getSource() == blueValue)) {
      Color c = getColor();
      if (c != null) {
	selectedColor.setBackground(c);
      }
      else {
	Toolkit.getDefaultToolkit().beep();
      }
    }
    else if (e.getSource() == ocp.ok) {
            
      Color c = getColor();
      if (c != null) {
	setVisible(false);
	Victor v = (Victor) getParent();
	v.tp.currentColor = c;
	if (v.s.setShapeColor(v.tp.currentColor)) {
	  v.s.repaint();
	}
	v.tp.colorTool.setBackground(v.tp.currentColor);
	dispose();
      }
      else {
	Toolkit.getDefaultToolkit().beep();
      }
      
    }
    else if (e.getSource() == ocp.cancel) {
      setVisible(false);
      dispose();
    }
    
  }


  public void mousePressed(MouseEvent e) {
  
    Color c = ((Component) e.getSource()).getBackground();
    selectedColor.setBackground(c);
    
    redValue.setText(String.valueOf(c.getRed()));
    greenValue.setText(String.valueOf(c.getGreen()));
    blueValue.setText(String.valueOf(c.getBlue()));
    
  }

  
  public void mouseClicked(MouseEvent e) {}
  public void mouseReleased(MouseEvent e) {}
  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}

}

