import java.awt.*;
import java.io.*;


class Ellipse extends Shape {

	boolean fill;
	

	public Ellipse(Sheet s, Color c, int x, int y, int width, int height, boolean fill) {
		
		this.s = s;
		color = c;

		boundingBox = new Rectangle();
		boundingBox.x = x;
		boundingBox.y = y;
		boundingBox.width = width;
		boundingBox.height = height;

		this.fill = fill;

	}

	
	public Ellipse(Sheet s, Point p1, Point p2, Color c, boolean fill) {
		
		this.s = s;
		color = c;
		boundingBox = Geometry.createRectangle(p1, p2);
		this.fill = fill;
		
	}


	public Ellipse(Sheet s, BufferedReader in) {
		
		this.s = s;
		color = InOut.readColor(in);
		boundingBox = InOut.readRectangle(in);
		fill = InOut.readBoolean(in);
		
		try {
			in.readLine();	// skip blank line
		}
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }
    
	}


	protected Object clone() {
		
		return new Ellipse(s, color,
											 boundingBox.x, boundingBox.y, boundingBox.width, boundingBox.height,
											 fill);
		
	}

	
	public String toString() {
		
		String s;

		s = "Ellipse\n";
		s = s + InOut.writeColor(color);
		s = s + InOut.writeRectangle(boundingBox);
		s = s + InOut.writeBoolean(fill) + "\n";
		
		return s;
		
	}

	
	void draw(Graphics g) {
		
		g.setColor(color);

		if (boundingBox.isEmpty()) {
			Point o = Geometry.opposite(new Point(boundingBox.x, boundingBox.y), boundingBox);
			g.drawLine(boundingBox.x, boundingBox.y, o.x, o.y);
		}
		else if (fill) {
			g.fillOval(boundingBox.x, boundingBox.y, boundingBox.width, boundingBox.height);
		}
		else {
			g.drawOval(boundingBox.x, boundingBox.y, boundingBox.width - 1, boundingBox.height - 1);
		}
		
	}


	boolean contains(int clickedX, int clickedY) {

		int dCF = (int) Math.round(Math.sqrt(Math.abs(
								Math.pow(boundingBox.width / 2, 2) - Math.pow(boundingBox.height / 2, 2))));
		Point c = new Point(boundingBox.x + boundingBox.width / 2,
												boundingBox.y + boundingBox.height / 2);

		Point f1;
		Point f2;
		
		if (boundingBox.width > boundingBox.height) {
			f1 = new Point(c.x - dCF, c.y);
			f2 = new Point(c.x + dCF, c.y);
		}
		else {
			f1 = new Point(c.x, c.y - dCF);
			f2 = new Point(c.x, c.y + dCF);
		}

		Point p = new Point(clickedX, clickedY);
		double dF1P = Geometry.distance(f1, p);
		double dPF2 = Geometry.distance(p, f2);

		if (fill) {
			return Math.round(dF1P + dPF2) <= Math.max(boundingBox.width, boundingBox.height);
		}
		else {
			return Math.abs(dF1P + dPF2 - Math.max(boundingBox.width, boundingBox.height)) <= 2;
		}
		
	}


	void translate(int dx, int dy) {
		
		boundingBox.translate(dx, dy);
		
		if (selected) {
			removeHandles();
			createHandles();
		}
		
	}


	void rotate(Point c) {

		Point tl = boundingBox.getLocation();
		Point br = Geometry.opposite(tl, boundingBox);
		
		Point p1 = Geometry.rotate90CW(tl, c);
		Point p2 = Geometry.rotate90CW(br, c);

		boundingBox = Geometry.createRectangle(p1, p2);
		
	}


	void scale(Shape initialShape, Point o, float xScale, float yScale) {

		Point tl = initialShape.boundingBox.getLocation();
		Point br = Geometry.opposite(tl, initialShape.boundingBox);
		
		Point p1 = Geometry.scale(tl, o, xScale, yScale);
		Point p2 = Geometry.scale(br, o, xScale, yScale);

		boundingBox = Geometry.createRectangle(p1, p2);
		
	}

	
	void reshape(Shape initialShape, Point start, Point end) {
		
		resizeBoundingBox(initialShape.boundingBox, start, end);
				
	}
	
}

