import java.awt.*;


public class Geometry {

  public static double distance(Point p1, Point p2) {
    
    return Math.sqrt(Math.pow((p1.x - p2.x), 2) + Math.pow((p1.y - p2.y), 2));
    
  }


  public static boolean segmentContains(Point p1, Point p2, Point p) {
		
    if (Math.round(distance(p1, p) + distance(p, p2)) >
	Math.round(distance(p1, p2))) {
      return false;
    }
    else {
      return true;
    }
		
  }

	
  public static Point constrainPoint(Point p, Point refPoint,
				     Rectangle initialBBox) {

    Point endPoint = new Point(p);
		
    if (!p.equals(refPoint)) {
      int dx = p.x - refPoint.x;
      int dy = p.y - refPoint.y;
      int max = Math.max(Math.abs(dx), Math.abs(dy));
      int min = Math.min(Math.abs(dx), Math.abs(dy));

      float xDirection = 1;
      if (dx < 0) {
	xDirection = -1;
      }

      float yDirection = 1;
      if (dy < 0) {
	yDirection = -1;
      }	

      int refMax = Math.max(initialBBox.width, initialBBox.height);
      int refMin = Math.min(initialBBox.width, initialBBox.height);
      float refRatio;
      if (refMax != 0) {
	refRatio = ((float) refMin) / ((float) refMax);
      }
      else {
	refRatio = 1;
      }
			
      Point o = opposite(refPoint, initialBBox);
      int refDx = refPoint.x - o.x;
      int refDy = refPoint.y - o.y;

      double refAngle;
      if ((initialBBox.width == 0) && (initialBBox.height == 0)) {
	refAngle = Math.PI / 4;
      }
      else {
	refAngle = Math.atan2(initialBBox.height, initialBBox.width);
      }
      double inf = refAngle / 2;
      double sup = inf + Math.PI / 4;
      double angle = Math.atan2(Math.abs(dy), Math.abs(dx));

      int refXDirection;
      if (refDx == 0) {
	refXDirection = 0;
      }
      else {
	refXDirection = refDx / Math.abs(refDx);
      }

      int refYDirection;
      if (refDy == 0) {
	refYDirection = 0;
      }
      else {
	refYDirection = refDy / Math.abs(refDy);
      }
			
      if ((angle > inf) && (angle < sup) &&
	  ((xDirection * yDirection *	refXDirection * refYDirection) >= 0)) {

	if (refMax != 0) {
	  if (initialBBox.width < initialBBox.height) {
	    xDirection *= refRatio;
	  }
	  else {
	    yDirection *= refRatio;
	  }
	}
      }
      else {
	if (Math.abs(dx) == min) {
	  // The segment will be vertical
	  xDirection = 0;
	}
	else {
	  // The segment will be horizontal
	  yDirection = 0;
	}
      }

      endPoint.x = refPoint.x + Math.round(xDirection * max);
      endPoint.y = refPoint.y + Math.round(yDirection * max);
    }

    return endPoint;
  }


  public static Point createCenter(Rectangle r) {
    
    return new Point(r.x + r.width / 2, r.y + r.height / 2);
    
  }

	
  public static Point opposite(Point p, Rectangle r) {

    Point o = new Point();

    if ((p.x == r.x) && (r.width != 0)) {
      o.x = p.x + r.width - 1;
    }
    else {
      o.x = r.x;
    }

    if ((p.y == r.y) && (r.height != 0)) {
      o.y = p.y + r.height - 1;
    }
    else {
      o.y = r.y;
    }

    return o;
  }


  public static Rectangle createRectangle(Point p1, Point p2) {

    Rectangle boundingBox;
		
    boundingBox = new Rectangle(new Point(Math.min(p1.x, p2.x),
					  Math.min(p1.y, p2.y)));

    if (p1.x == p2.x) {
      boundingBox.width = 0;
    }
    else {
      boundingBox.width = Math.abs(p1.x - p2.x) + 1;
    }

    if (p1.y == p2.y) {
      boundingBox.height = 0;
    }
    else {
      boundingBox.height = Math.abs(p1.y - p2.y) + 1;
    }

    return boundingBox;
  }


  public static Point rotate90CW(Point p, Point c) {

    return new Point(c.x - p.y + c.y, c.y + p.x - c.x);
		
  }

	
  public static Point scale(Point p1, Point o, float xScale, float yScale) {

    Point p2 = new Point();
		
    p2.x = Math.round(xScale * (p1.x - o.x)) + o.x;
    p2.y = Math.round(yScale * (p1.y - o.y)) + o.y;

    return p2;
		
  }

}
