import java.awt.*;
import java.awt.event.*;


class GraphicButton extends Canvas {

  private byte type;
  private boolean state;
  private Image buttonImage;

  public static final byte empty       = 0;
  public static final byte image       = 1;
  public static final byte line        = 2;
  public static final byte drawRect    = 3;
  public static final byte fillRect    = 4;
  public static final byte drawOval    = 5;
  public static final byte fillOval    = 6;
  public static final byte drawPolygon = 7;
  public static final byte fillPolygon = 8;
  public static final byte text        = 9;
  


  public GraphicButton(byte type, boolean state) {
    
    this.type = type;
    this.state = state;
    
  }


  public GraphicButton(byte type, String imagePath, boolean state) {
    
    this.type = type;
    buttonImage = (Toolkit.getDefaultToolkit()).getImage(imagePath);
    this.state = state;
    
  }
  

  public byte getType() {

    return type;

  }


  public void setType(byte type) {

    this.type = type;
    repaint();

  }

  
  public boolean getState() {

    return state;

  }


  public void setState(boolean state) {

    this.state = state;
    repaint();

  }


  void drawCenteredString(Graphics g, String str, Font f, int x, int y) {
    
    g.setFont(f);
    FontMetrics fm = g.getFontMetrics(f);

    // Calcul des coordonnes  partir desquelles il faut afficher la chane
    // pour qu'elle soit centre autour du point (x, y) :
    int xstart = x - (fm.stringWidth(str) / 2);
    int ystart = y + (fm.getAscent() / 2);

    g.drawString(str, xstart, ystart);
    
  }

  
  public void paint(Graphics g) {

    Dimension d = getSize();

    if (state == true) {
      // L'aspect 3D n'est pas visible...
      g.draw3DRect(0, 0, d.width - 1, d.height - 1, false);
    }
		
    switch(type) {
      
    case image:
      int x = (d.width / 2) - (buttonImage.getWidth(this) / 2);
      int y = (d.height / 2) - (buttonImage.getHeight(this) / 2);
      g.drawImage(buttonImage, x, y, this);
      break;

    case line:
      g.drawLine(5, d.height - 6, d.width - 6, 5);
      break;

    case drawRect:
      g.drawRect(5, 5, d.width - 11, d.height - 11);
      break;

    case fillRect:
      g.fillRect(5, 5, d.width - 10, d.height - 10);
      break;
      
    case drawOval:
      g.drawOval(5, 5, d.width - 11, d.height - 11);
      break;

    case fillOval:
      g.fillOval(5, 5, d.width - 10, d.height - 10);
      break;
      
    case drawPolygon:
    case fillPolygon:
      int xpoints[] = {
	5,
	5 + d.width / 4,
	d.width - 6,
	d.width - 6 - d.width / 6,
	5 + d.width / 7
      };
      
      int ypoints[] = {
	5,
	5 + d.height / 5,
	5 + d.height / 8,
	d.height - 6 - d.height / 4,
	d.height - 6
      };

      if (type == drawPolygon) {
	g.drawPolygon(xpoints, ypoints, 5);
      }
      else {
	g.fillPolygon(xpoints, ypoints, 5);
      }
      break;

    case text:
      //g.drawString("T", 5, d.height - 6);

      // La taille de la fonte est calcule en fonction
      // de la taille du bouton :
      Font f = new Font("TimesRoman", Font.BOLD, d.height);

      drawCenteredString(g, "T", f, d.width / 2, d.height / 2);
      break;

    }
		
  }

}
