import java.awt.*;
import java.io.*;


class Group extends Shape {

	Shape[] shapes;


	public Group(Sheet s, Shape[] shapes) {
		this.s = s;
		this.shapes = shapes;
		boundingBox = Shape.createBoundingBox(shapes);
	}


	public Group(Sheet s, BufferedReader in) {

		this.s = s;
		
		int nbShapes = 0;
		Shape[] shapes;
		
		try {
			nbShapes = Integer.parseInt(in.readLine());
		}
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }

		shapes = new Shape[nbShapes];
		
    for (int i = 0 ; i < nbShapes ; i++) {
    	shapes[i] = InOut.readShape(s, in);
    }

		this.shapes = shapes;
		
    boundingBox = Shape.createBoundingBox(shapes);

    try {
			in.readLine();	// skip blank line
		}
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }
	}


	protected Object clone() {
		Shape[] shapesCopy = new Shape[shapes.length];

		for (int i = 0 ; i < shapes.length ; i++) {
			shapesCopy[i] = (Shape) shapes[i].clone();
		}
		
		return new Group(s, shapesCopy);
	}


	void setColor(Color c) {
		for (int i = 0 ; i < shapes.length ; i++) {
			shapes[i].setColor(c);
		}
	}

	
	public String toString() {
		
		String s;

		s = "Group\n";
		s = s + shapes.length + "\n";

		for (int i = 0 ; i < shapes.length ; i++) {
			s += shapes[i].toString();
		}
		
		s += "\n";
		
		return s;
	}


	void draw(Graphics g) {
		for (int i = 0 ; i < shapes.length ; i++) {
			shapes[i].draw(g);
		}
	}


	boolean contains(int x, int y) {
		
		for (int i = 0 ; i < shapes.length ; i++) {
			if (shapes[i].contains(x, y)) {
				return true;
			}
		}
		
		return false;
	}


	void translate(int dx, int dy) {
		
		boundingBox.translate(dx, dy);

		for (int i = 0 ; i < shapes.length ; i++) {
			shapes[i].translate(dx, dy);
		}

		if (selected) {
			removeHandles();
			createHandles();
		}
		
	}


	void rotate(Point c) {

		for (int i = 0 ; i < shapes.length ; i++) {
			shapes[i].rotate(c);
		}

		boundingBox = Shape.createBoundingBox(shapes);
		
	}


	void scale(Shape initialShape, Point o, float xScale, float yScale) {

		for (int i = 0 ; i < shapes.length ; i++) {
			shapes[i].scale(((Group) initialShape).shapes[i], o, xScale, yScale);
		}

		boundingBox = Shape.createBoundingBox(shapes);

	}

	
	void reshape(Shape initialShape, Point start, Point end) {
		
		Point o = Geometry.opposite(start, initialShape.boundingBox);
						
		float xRatio;
		if (start.x == o.x) {
			xRatio = 1;
		}
		else {
			xRatio = ((float) (end.x - o.x)) / (start.x - o.x);
		}
		
		float yRatio;
		if (start.y == o.y) {
			yRatio = 1;
		}
		else {
			yRatio = ((float) (end.y - o.y)) / (start.y - o.y);
		}

		scale(initialShape, o, xRatio, yRatio);

		byte xDir;
		if (o.x > initialShape.boundingBox.x) {
			// o is on the right side
			xDir = 1;
		}
		else {
			// o is on the left side
			xDir = -1;
		}

		if (xRatio < 0) {
			xDir *= -1;
		}

		int x;
		if ((xDir == -1) || (boundingBox.width == 0)) {
			x = boundingBox.x;
		}
		else {
			x = boundingBox.x + boundingBox.width - 1;
		}
		
		byte yDir;
		if (o.y > initialShape.boundingBox.y) {
			// o is on the bottom side
			yDir = 1;
		}
		else {
			// o is on the top side
			yDir = -1;
		}

		if (yRatio < 0) {
			yDir *= -1;
		}

		int y;
		if ((yDir == -1) || (boundingBox.height == 0)) {
			y = boundingBox.y;
		}
		else {
			y = boundingBox.y + boundingBox.height - 1;
		}

		int dx = o.x - x;
		int dy = o.y - y;
		
		translate(dx, dy);

	}


	void createHandles() {
		super.createHandles();
		s.setNbSelectedGroups(s.getNbSelectedGroups() + 1);
	}

}

