import java.awt.*;
import java.io.*;


class InOut {

  static Shape readShape(Sheet s, BufferedReader in) {

    String line = null;
    Shape shape = null;
		
    try {
      line = in.readLine();
    }
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }
    
    if (line == null) {
      return null;
    }
		
    if (line.equals("Line")) {
      shape = new Line(s, in);
    }
    else if (line.equals("MyRectangle")) {
      shape = new MyRectangle(s, in);
    }
    else if (line.equals("Ellipse")) {
      shape = new Ellipse(s, in);
    }
    else if (line.equals("Polygon")) {
      shape = new PolygonShape(s, in);
    }
    else if (line.equals("Text")) {
      shape = new TextShape(s, in);
    }
    else if (line.equals("Group")) {
      shape = new Group(s, in);
    }

    return shape;
  }
	
		
  static String writeColor(Color c) {
    
    return String.valueOf(c.getRGB()) + "\n";
    
  }


  static Color readColor(BufferedReader in) {

    String line = null;
		
    try {
      line = in.readLine();
    }
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }

    return Color.decode(line);
    
  }


  static String writeBoolean(boolean b) {

    if (b) {
      return "1\n";
    }
    else {
      return "0\n";
    }
		
  }


  static boolean readBoolean(BufferedReader in) {

    String line = null;
		
    try {
      line = in.readLine();
    }
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }

    if (line.equals("1")) {
      return true;
    }
    else {
      return false;
    }
		
  }


  static String writePoint(Point p) {
    
    return String.valueOf(p.x) + "\n" + String.valueOf(p.y) + "\n";
    
  }


  static Point readPoint(BufferedReader in) {

    int x = 0;
    int y = 0;
		
    try {
      x = Integer.parseInt(in.readLine());
      y = Integer.parseInt(in.readLine());
    }
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }
		
    return new Point(x, y);
    
  }


  static String writeRectangle(Rectangle r) {
    
    return String.valueOf(r.x) + "\n" +
      String.valueOf(r.y) + "\n" +
      String.valueOf(r.width) + "\n" +
      String.valueOf(r.height) + "\n";
    
  }


  static Rectangle readRectangle(BufferedReader in) {

    int x, y, width, height;
    x = y = width = height = 0;
		
    try {
      x = Integer.parseInt(in.readLine());
      y = Integer.parseInt(in.readLine());
      width = Integer.parseInt(in.readLine());
      height = Integer.parseInt(in.readLine());
    }
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }
		
    return new Rectangle(x, y, width, height);
    
  }


  static String writePolygon(Polygon p) {

    String s;

    s = String.valueOf(p.npoints) + "\n";

    for (int i = 0 ; i < p.npoints ; i++) {
      int x = p.xpoints[i];
      int y = p.ypoints[i];
      s += writePoint(new Point(x, y));
    }
		
    return s;
    
  }


  static Polygon readPolygon(BufferedReader in) {

    Polygon p = new Polygon();
    int nbPoints = 0;

    try {
      nbPoints = Integer.parseInt(in.readLine());
    }
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }
		
    for (int i = 0 ; i < nbPoints ; i++) {
      Point point = readPoint(in);
      p.addPoint(point.x, point.y);
    }

    return p;
    
  }


  static String writeFont(Font f) {

    String s;

    s = f.getName() + "\n";
    s = s + String.valueOf(f.getStyle()) + "\n";
    s = s + String.valueOf(f.getSize()) + "\n";
		
    return s;
    
  }


  static Font readFont(BufferedReader in) {

    String name = "Serif";
    int style = Font.PLAIN;
    int size = 12;
		
    try {
      name = in.readLine();
      style = Integer.parseInt(in.readLine());
      size = Integer.parseInt(in.readLine());
    }
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }
		
    return new Font(name, style, size);
    
  }
		
}
