import java.awt.*;
import java.io.*;


class Line extends Shape {

	private Point p1, p2;


	public Line(Sheet s, Point p1, Point p2, Color c) {
		this.s = s;
		color = c;
		this.p1 = p1;
		this.p2 = p2;
		boundingBox = Geometry.createRectangle(p1, p2);
	}


	public Line(Sheet s, BufferedReader in) {
		this.s = s;
		color = InOut.readColor(in);
		p1 = InOut.readPoint(in);
		p2 = InOut.readPoint(in);
		boundingBox = Geometry.createRectangle(p1, p2);

		try {
			in.readLine();	// skip blank line
		}
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }
	}


	protected Object clone() {
		return new Line(s, new Point(p1), new Point(p2), color);
	}

	
	public String toString() {
		String s;

		s = "Line\n";
		s = s + InOut.writeColor(color);
		s = s + InOut.writePoint(p1);
		s = s + InOut.writePoint(p2) + "\n";
		
		return s;
	}

	
	void draw(Graphics g) {
		g.setColor(color);
		g.drawLine(p1.x, p1.y, p2.x, p2.y);
	}


	boolean contains(int x, int y) {
		Point p = new Point(x, y);
		return Geometry.segmentContains(p1, p2, p);
	}


	void translate(int dx, int dy) {
		boundingBox.translate(dx, dy);
		
		p1.translate(dx, dy);
		p2.translate(dx, dy);

		if (selected) {
			removeHandles();
			createHandles();
		}
	}


	void rotate(Point c) {
		
		p1 = Geometry.rotate90CW(p1, c);
		p2 = Geometry.rotate90CW(p2, c);

		boundingBox = Geometry.createRectangle(p1, p2);
		
	}


	void scale(Shape initialShape, Point o, float xScale, float yScale) {

		p1 = Geometry.scale(((Line) initialShape).p1, o, xScale, yScale);
		p2 = Geometry.scale(((Line) initialShape).p2, o, xScale, yScale);

		boundingBox = Geometry.createRectangle(p1, p2);
		
	}

	
	void reshape(Shape initialShape, Point start, Point end) {

		resizeBoundingBox(initialShape.boundingBox, start, end);

		if (((Line) initialShape).p1.equals(start)) {
			p1 = end;
		}
		else {
			p2 = end;
		}
		
	}

	
	void createHandles() {
		
		selected = true;
		
		s.handleList.addElement(new Handle(this, p1));
		
		if (!p2.equals(p1)) {
			s.handleList.addElement(new Handle(this, p2));
		}

		s.setNbSelectedShapes(s.getNbSelectedShapes() + 1);
		
	}

}

