import java.awt.*;
import java.io.*;


class MyRectangle extends Shape {

  boolean fill;
	

  public MyRectangle(Sheet s, Point p1, Point p2, Color c, boolean fill) {

    this.s = s;
    color = c;
    boundingBox = Geometry.createRectangle(p1, p2);

    this.fill = fill;
		
  }


  public MyRectangle(Sheet s, Rectangle r, Color c, boolean fill) {

    this.s = s;
    color = c;
    boundingBox = r;

    this.fill = fill;
		
  }


  public MyRectangle(Sheet s, BufferedReader in) {
		
    this.s = s;
    color = InOut.readColor(in);
    boundingBox = InOut.readRectangle(in);
    fill = InOut.readBoolean(in);
		
    try {
      in.readLine();	// skip blank line
    }
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }
    
  }


  protected Object clone() {
		
    return new MyRectangle(s, new Rectangle(boundingBox), color, fill);
		
  }

	
  public String toString() {
		
    String s;

    s = "MyRectangle\n";
    s = s + InOut.writeColor(color);
    s = s + InOut.writeRectangle(boundingBox);
    s = s + InOut.writeBoolean(fill) + "\n";
		
    return s;
		
  }

	
  void draw(Graphics g) {
		
    g.setColor(color);
		
    if (boundingBox.isEmpty()) {
      Point o = Geometry.opposite(new Point(boundingBox.x, boundingBox.y),
				  boundingBox);
      g.drawLine(boundingBox.x, boundingBox.y, o.x, o.y);
    }
    else if (fill) {
      g.fillRect(boundingBox.x, boundingBox.y,
		 boundingBox.width, boundingBox.height);
    }
    else {
      g.drawRect(boundingBox.x, boundingBox.y,
		 boundingBox.width - 1, boundingBox.height - 1);
    }
		
  }


  boolean contains(int x, int y) {

    Point p = new Point(x, y);
		
    if (boundingBox.isEmpty()) {
      Point p1 = new Point(boundingBox.x, boundingBox.y);
      Point p2 = Geometry.opposite(p1, boundingBox);
      return Geometry.segmentContains(p1, p2, p);
    }
    else if (fill) {
      return boundingBox.contains(p);
    }
    else {
      Point p1 = new Point(boundingBox.x, boundingBox.y);
      Point p2 = new Point(boundingBox.x + boundingBox.width - 1,
			   boundingBox.y);
      Point p3 = new Point(boundingBox.x + boundingBox.width - 1,
			   boundingBox.y + boundingBox.height - 1);
      Point p4 = new Point(boundingBox.x,
			   boundingBox.y + boundingBox.height - 1);

      return
	Geometry.segmentContains(p1, p2, p) ||
	Geometry.segmentContains(p2, p3, p) ||
	Geometry.segmentContains(p3, p4, p) ||
	Geometry.segmentContains(p4, p1, p);
    }
		
  }


  void translate(int dx, int dy) {
		
    boundingBox.translate(dx, dy);

    if (selected) {
      removeHandles();
      createHandles();
    }
		
  }


  void rotate(Point c) {

    Point tl = boundingBox.getLocation();
    Point br = Geometry.opposite(tl, boundingBox);
		
    Point p1 = Geometry.rotate90CW(tl, c);
    Point p2 = Geometry.rotate90CW(br, c);

    boundingBox = Geometry.createRectangle(p1, p2);
		
  }


  void scale(Shape initialShape, Point o, float xScale, float yScale) {

    Point tl = initialShape.boundingBox.getLocation();
    Point br = Geometry.opposite(tl, initialShape.boundingBox);
		
    Point p1 = Geometry.scale(tl, o, xScale, yScale);
    Point p2 = Geometry.scale(br, o, xScale, yScale);

    boundingBox = Geometry.createRectangle(p1, p2);
				
  }

	
  void reshape(Shape initialShape, Point start, Point end) {
		
    resizeBoundingBox(initialShape.boundingBox, start, end);
		
  }
	
}
