import java.awt.*;
import java.util.Vector;


abstract class Shape {

  Sheet s;
  boolean selected = false;
  Color color;
  Rectangle boundingBox;

  protected abstract Object clone();
  abstract void draw(Graphics g);
  abstract boolean contains(int x, int y);
  abstract void translate(int dx, int dy);
  abstract void rotate(Point c);
  abstract void scale(Shape initialShape, Point o, float xScale, float yScale);
  abstract void reshape(Shape initialShape, Point start, Point end);
	
	
  void setColor(Color c) {
    
    color = c;
    
  }


  void createHandles() {
		
    selected = true;
		
    Point p1 = new Point(boundingBox.x, boundingBox.y);
    s.handleList.addElement(new Handle(this, p1));

    if (boundingBox.width != 0) {
      Point p2 = new Point(boundingBox.x + boundingBox.width - 1,
			   boundingBox.y);
      s.handleList.addElement(new Handle(this, p2));
    }

    if ((boundingBox.width != 0) && (boundingBox.height != 0)) {
      Point p3 = new Point(boundingBox.x + boundingBox.width - 1,
			   boundingBox.y + boundingBox.height - 1);
      s.handleList.addElement(new Handle(this, p3));
    }

    if (boundingBox.height != 0) {
      Point p4 = new Point(boundingBox.x,
			   boundingBox.y + boundingBox.height - 1);
      s.handleList.addElement(new Handle(this, p4));
    }

    s.setNbSelectedShapes(s.getNbSelectedShapes() + 1);
		
  }

	
  void removeHandles() {
    
    selected = false;
		
    int i = 0 ;
    while (i < s.handleList.size()) {
      if (((Handle) s.handleList.elementAt(i)).owner == this) {
	s.handleList.removeElementAt(i);
      } else {
	i++;
      }
    }

    if (this instanceof Group) {
      s.setNbSelectedGroups(s.getNbSelectedGroups() - 1);
    }
    s.setNbSelectedShapes(s.getNbSelectedShapes() - 1);
    
  }


  void resizeBoundingBox(Rectangle initialBoundingBox,
			 Point start, Point end) {
		
    Point o = Geometry.opposite(start, initialBoundingBox);
    boundingBox = Geometry.createRectangle(o, end);
    
  }

	
  static Rectangle createBoundingBox(Shape[] shapes) {
		
    int minX = shapes[0].boundingBox.x;
    int minY = shapes[0].boundingBox.y;
		
    int maxX;
    if (shapes[0].boundingBox.width == 0) {
      maxX = minX;
    }
    else {
      maxX = minX + shapes[0].boundingBox.width - 1;
    }

    int maxY;
    if (shapes[0].boundingBox.height == 0) {
      maxY = minY;
    }
    else {
      maxY = minY + shapes[0].boundingBox.height - 1;
    }

    for (int i = 1 ; i < shapes.length ; i++) {
      Rectangle bBox = shapes[i].boundingBox;

      minX = Math.min(minX, bBox.x);
      minY = Math.min(minY, bBox.y);

      if (bBox.width == 0) {
	maxX = Math.max(maxX, bBox.x);
      }
      else {
	maxX = Math.max(maxX, bBox.x + bBox.width - 1);
      }

      if (bBox.height == 0) {
	maxY = Math.max(maxY, bBox.y);
      }
      else {
	maxY = Math.max(maxY, bBox.y + bBox.height - 1);
      }
    }

    int width;
    if (maxX == minX) {
      width = 0;
    }
    else {
      width = maxX - minX + 1;
    }

    int height;
    if (maxY == minY) {
      height = 0;
    }
    else {
      height = maxY - minY + 1;
    }
		
    return new Rectangle(minX, minY, width, height);
    
  }

}
