import java.awt.*;
import java.awt.image.*;
import java.util.*;
import java.awt.event.*;
import java.io.*;


class Sheet extends Canvas {

  Victor v;
  Dimension d;
  private String filePath, fileName;
  Grid grid;
  Vector shapeList = new Vector();
  Vector clipBoard = new Vector();
  private int nbSelectedShapes;
  private int nbSelectedGroups;
  Vector handleList = new Vector();
  boolean areaSelection;
  Point anchor;  // start of current shape
  Point currentPoint; // current end of shape
  Polygon currentPolygon;
  SheetKeyListener skl;
  SheetMouseListener sml;
  SheetMouseMotionListener smml;

  
  public Sheet(Victor v) {
		
    this.v = v;
    d = new Dimension(640, 480);
    grid = new Grid(5);
    nbSelectedShapes = 0;
    areaSelection =  false;
    currentPolygon = new Polygon();
		
    setBackground(Color.white);

    skl = new SheetKeyListener(this);
    addKeyListener((KeyListener) skl);
    
    sml = new SheetMouseListener(this);
    addMouseListener((MouseListener) sml);

    smml = new SheetMouseMotionListener(this);
    addMouseMotionListener((MouseMotionListener) smml);
    
    setSize(d);
    
  }


  public void setFilePath(String filePath) {

    this.filePath = filePath;

  }


  public String getFilePath() {

    return filePath;

  }


  public void setFileName(String fileName) {

    this.fileName = fileName;
    if (fileName != null) {
      v.setTitle(v.title + ": " + fileName);
    }
    else {
      v.setTitle(v.title);
    }

  }


  public String getFileName() {

    return fileName;

  }

  
  void setNbSelectedShapes(int val) {
		
    nbSelectedShapes = val;

    // At least 2 shapes needed to make a group:
    if (val < 2) {
      v.align.setEnabled(false);
      v.group.setEnabled(false);
    }
    else {
      v.align.setEnabled(true);
      v.group.setEnabled(true);
    }

    if (val == 0) {
      setNbSelectedGroups(0);
			
      v.cut.setEnabled(false);
      v.copy.setEnabled(false);
      v.delete.setEnabled(false);
	 		
      v.foreground.setEnabled(false);
      v.closer.setEnabled(false);
      v.farther.setEnabled(false);
      v.background.setEnabled(false);
      v.rotate.setEnabled(false);
      v.hMirror.setEnabled(false);
      v.vMirror.setEnabled(false);
    }
    else {
      v.cut.setEnabled(true);
      v.copy.setEnabled(true);
      v.delete.setEnabled(true);
	 		
      v.foreground.setEnabled(true);
      v.closer.setEnabled(true);
      v.farther.setEnabled(true);
      v.background.setEnabled(true);
      v.rotate.setEnabled(true);
      v.hMirror.setEnabled(true);
      v.vMirror.setEnabled(true);
    }
  }

	
  int getNbSelectedShapes() {
    return nbSelectedShapes;
  }

	
  void setNbSelectedGroups(int val) {
    nbSelectedGroups = val;

    if (val == 0) {
      // Nothing to ungroup
      v.ungroup.setEnabled(false);
    }
    else {
      v.ungroup.setEnabled(true);
    }
  }

	
  int getNbSelectedGroups() {
    return nbSelectedGroups;
  }


  Shape getCurrentShape(int x, int y) {

    Shape d = null;
    Point endPoint = new Point(x, y);

    if ((anchor != null) && skl.isShiftDown()) {
      endPoint = Geometry.constrainPoint(endPoint, anchor,
					 new Rectangle(anchor,
						       new Dimension(1, 1)));
    }

    if (v.snapToGrid.getState() == true) {
      endPoint = grid.snap(endPoint);
    }

    switch (v.tp.currentTool) {

    case ToolPanel.line:
      d = new Line(this, anchor, endPoint, v.tp.currentColor);
      break;

    case ToolPanel.rectangle:
      d = new MyRectangle(this, anchor, endPoint, v.tp.currentColor,
			  v.tp.fill);
      break;

    case ToolPanel.ellipse:
      d = new Ellipse(this, anchor, endPoint, v.tp.currentColor, v.tp.fill);
      break;

    case ToolPanel.polygon:
      d = new PolygonShape(this, currentPolygon, v.tp.currentColor, v.tp.fill);
      break;

    }

    return d;
  }
	
		
  void addShape(int x, int y) {

    currentPoint = null;

    Shape s = getCurrentShape(x, y);
  	
    if ((s instanceof PolygonShape) && (currentPolygon.npoints < 3)) {
      repaint();
      return;
    }
  	
    shapeList.addElement(s);
    repaint();
  }


  void nothingSelected() {
		
    setNbSelectedShapes(0);
    handleList.removeAllElements();
		
  }

 			
  int deleteShapes() {
		
    int nbDeletedShapes = 0;
		
    int i = 0 ;
    while (i < shapeList.size()) {
      if (((Shape) shapeList.elementAt(i)).selected) {
	shapeList.removeElementAt(i);
	nbDeletedShapes++;
      }
      else {
	i++;
      }
    }

    if (nbDeletedShapes != 0) {
      nothingSelected();
    }

    return nbDeletedShapes;
		
  }


  void deleteAllShapes() {
    
    shapeList.removeAllElements();
    nothingSelected();
    
  }
	

  int translateShapes(int dx, int dy) {
		
    int nbTranslatedShapes = 0;

    for (int i = 0 ; i < shapeList.size() ; i++) {
      Shape shape = (Shape) shapeList.elementAt(i);
      if (shape.selected) {
	shape.translate(dx, dy);
	nbTranslatedShapes++;
      }
    }

    return nbTranslatedShapes;
		
  }


  void unselectShapes() {
		
    for (int i = 0 ; i < shapeList.size() ; i++) {
      ((Shape) shapeList.elementAt(i)).selected = false;
    }

    nothingSelected();
		
  }


  Shape findShape(int x, int y) {

    // Is there a Handle there?
    for (int i = handleList.size() - 1 ; i >= 0 ; i--) {
      Shape shape = (Shape) handleList.elementAt(i);
      if (shape.contains(x, y)) {
	return shape;
      }
    }
		
    // Or is there a Shape that's not a Handle there?
    for (int i = shapeList.size() - 1 ; i >= 0 ; i--) {
      Shape shape = (Shape) shapeList.elementAt(i);
      if (shape.contains(x, y)) {
	return shape;
      }
    }

    return null;	// Nothing found there...
  }


  void areaSelection(Rectangle selectRectangle, boolean shiftDown) {
		
    for (int i = 0 ; i < shapeList.size() ; i++) {
      Shape shape = (Shape) shapeList.elementAt(i);
      Rectangle intersection = selectRectangle.intersection(shape.boundingBox);
      if (intersection.equals(shape.boundingBox)) {
	// selectRectangle contains shape
	if (shiftDown) {
	  if (shape.selected) {
	    shape.removeHandles();
	  }
	  else {
	    shape.createHandles();
	  }
	}
	else {
	  if (!shape.selected) {
	    shape.createHandles();
	  }
	}
      }
      else {
	if (!shiftDown && shape.selected) {
	  shape.removeHandles();
	}
      }
    }
		
  }

	
  boolean setShapeColor(Color c) {

    if (handleList.size() == 0) {
      return false;
    }
	
    for (int i = 0 ; i < shapeList.size() ; i++) {
      Shape shape = (Shape) shapeList.elementAt(i);
      if (shape.selected == true) {
	shape.setColor(c);
      }
    }

    return true;
  }
	

  boolean changeFillState() {
		
    int i = 0;
    int nbSelectedShapesFound = 0;
    int nbProcessedShapes = 0;
		
    while (nbSelectedShapesFound < getNbSelectedShapes()) {
      Shape shape = (Shape) shapeList.elementAt(i);
      if (shape.selected) {
	nbSelectedShapesFound++;
	if (shape instanceof MyRectangle) {
	  MyRectangle fillableShape = (MyRectangle) shape;
	  fillableShape.fill = !fillableShape.fill;
	  nbProcessedShapes++;
	}
	if (shape instanceof Ellipse) {
	  Ellipse fillableShape = (Ellipse) shape;
	  fillableShape.fill = !fillableShape.fill;
	  nbProcessedShapes++;
	}
	if (shape instanceof PolygonShape) {
	  PolygonShape fillableShape = (PolygonShape) shape;
	  fillableShape.fill = !fillableShape.fill;
	  nbProcessedShapes++;
	}
      }
      i++;
    }
				
    return (nbProcessedShapes > 0);
		
  }


  void load(String filePath, String fileName) {

    Shape shape;

    setFilePath(filePath);
    setFileName(fileName);
    
    try {
      BufferedReader in = new BufferedReader(
			    new InputStreamReader(
			      new FileInputStream(filePath + fileName)));

      deleteAllShapes();

      try {
 	d.width = Integer.parseInt(in.readLine());
 	d.height = Integer.parseInt(in.readLine());
	v.vcl.resizeSheet();
      }
      catch (IOException e) {
	System.out.println("IO Error:" + e.getMessage());
      }
      
      do {
	shape = InOut.readShape(this, in);
	if (shape != null) {
	  shapeList.addElement(shape);
	}
      } while (shape != null);
			
      in.close();
    }
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }

    repaint();
    
  }
	

  void save(String filePath, String fileName) {

    setFilePath(filePath);
    setFileName(fileName);
    
    try {
      PrintWriter out = new PrintWriter(
			  new BufferedOutputStream(
			    new FileOutputStream(filePath + fileName)));
      out.println(d.width);
      out.println(d.height);
      
      for (int i = 0 ; i < shapeList.size() ; i++) {
	out.print(shapeList.elementAt(i));
      }

      out.close();
    }
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }
    
  }
	

//   void export(String fileName) {
//     try {
//       PrintWriter out = new PrintWriter(
// 			  new BufferedOutputStream(
// 			    new FileOutputStream(fileName)));
		
//       Image i = createImage(d.width, d.height);
//       Graphics g = i.getGraphics();

//       for (int j = 0 ; j < shapeList.size() ; j++) {
// 	((Shape) shapeList.elementAt(j)).draw(g);
//       }
		
//       ImageProducer ip = i.getSource();
//       ip.startProduction(new BitmapExporter(ip, out));
//     }
//     catch (IOException e) {
//       System.out.println("IO Error:" + e.getMessage());
//     }
//   }


  void print() {
    PrintJob pj = Toolkit.getDefaultToolkit().getPrintJob(v, "",
							  new Properties());
    if (pj != null) {
      Graphics g = pj.getGraphics();
		
      // Draw existing shapes
      for (int i = 0 ; i < shapeList.size() ; i++) {
	((Shape) shapeList.elementAt(i)).draw(g);
      }

      pj.end();
    }
  }

	
  boolean copyShapes(boolean cut) {

    if (handleList.size() == 0) {
      return false;
    }
	
    clipBoard.removeAllElements();

    int i = 0 ;
    while (i < shapeList.size()) {
      Shape shape = (Shape) shapeList.elementAt(i);
      if (shape.selected == true) {
	clipBoard.addElement(shape.clone());
	if (cut) {
	  shapeList.removeElementAt(i);
	} else {
	  i++;
	}
      } else {
	i++;
      }
    }

    if (cut) {
      nothingSelected();
    }

    v.paste.setEnabled(true);
		
    return true;
  }

	
  boolean pasteShapes() {

    int size = clipBoard.size();

    if (size == 0) {
      return false;
    }

    unselectShapes();
		
    for (int i = 0 ; i < size ; i++) {
      Shape s = (Shape) ((Shape) clipBoard.elementAt(i)).clone();

      // Si l'objet que l'on s'apprte  coller sort de la feuille
      // (s'il provient d'une feuille plus grande), on le recentre :
      Point shapeC = Geometry.createCenter(s.boundingBox);
      if (!contains(shapeC)) {
	Point sheetC = new Point(d.width / 2, d.height / 2);
	s.translate(sheetC.x - shapeC.x, sheetC.y - shapeC.y);
      }
      
      shapeList.addElement(s);
      s.createHandles();
    }

    return true;
  }


  boolean selectAll() {

    int i = 0;
    int initialNbSelectedShapes = getNbSelectedShapes();
    int nbShapes = shapeList.size();

    while (getNbSelectedShapes() < nbShapes) {
      Shape shape = (Shape) shapeList.elementAt(i);
      if (!shape.selected) {
	shape.createHandles();
      }
      i++;
    }

    return (getNbSelectedShapes() > initialNbSelectedShapes);
		
  }

	
  boolean foreground() {
		
    int i = 0;
    int nbProcessedShapes = 0;
		
    while (nbProcessedShapes < getNbSelectedShapes()) {
      Shape shape = (Shape) shapeList.elementAt(i);
      if (shape.selected == true) {
	shapeList.removeElementAt(i);
	shapeList.addElement(shape);
	nbProcessedShapes++;
      }
      else {
	i++;
      }
    }
				
    return (nbProcessedShapes > 0);
  }


  boolean closer() {
		
    int i = shapeList.size() - 1;
    int nbSelectedShapesFound = 0;
    Shape previousShape = null;
		
    while (nbSelectedShapesFound < getNbSelectedShapes()) {
      Shape shape = (Shape) shapeList.elementAt(i);
      if (shape.selected == true) {
	nbSelectedShapesFound++;
	if (previousShape != null) {
	  if (previousShape.selected == true) {
	    previousShape = shape;
	  }
	  else {
	    Util.swap(shapeList, i, i + 1);
	  }
	}
	else {
	  previousShape = shape;
	}
      }
      else {
	previousShape = shape;
      }

      i--;
    }
				
    return (nbSelectedShapesFound > 0);
  }


  boolean farther() {
		
    int i = 0;
    int nbSelectedShapesFound = 0;
    Shape previousShape = null;
		
    while (nbSelectedShapesFound < getNbSelectedShapes()) {
      Shape shape = (Shape) shapeList.elementAt(i);
      if (shape.selected == true) {
	nbSelectedShapesFound++;
	if (previousShape != null) {
	  if (previousShape.selected == true) {
	    previousShape = shape;
	  }
	  else {
	    Util.swap(shapeList, i, i - 1);
	  }
	}
	else {
	  previousShape = shape;
	}
      }
      else {
	previousShape = shape;
      }

      i++;
    }
				
    return (nbSelectedShapesFound > 0);
  }

	
  boolean background() {
		
    int i = shapeList.size() - 1;
    int nbProcessedShapes = 0;
		
    while (nbProcessedShapes < getNbSelectedShapes()) {
      Shape shape = (Shape) shapeList.elementAt(i);
      if (shape.selected == true) {
	shapeList.removeElementAt(i);
	shapeList.insertElementAt(shape, 0);
	nbProcessedShapes++;
      }
      else {
	i--;
      }
    }
				
    return (nbProcessedShapes > 0);
  }


  boolean align(byte hAxis, byte vAxis) {

    if (((hAxis == AlignDialog.NONE) && (vAxis == AlignDialog.NONE)) ||
	(getNbSelectedShapes() < 2)) {
      return false;
    }

    int i = 0;
    int nbProcessedShapes = 0;
    Shape[] shapes = new Shape[getNbSelectedShapes()];
		
    while (nbProcessedShapes < getNbSelectedShapes()) {
      Shape shape = (Shape) shapeList.elementAt(i);
      if (shape.selected == true) {
	shapes[nbProcessedShapes] = shape;
	nbProcessedShapes++;
      }
      i++;
    }

    Rectangle bBox = Shape.createBoundingBox(shapes);

    int xOffset = bBox.width;
    int yOffset = bBox.height;
		
    if ((hAxis == AlignDialog.DISTRIBUTE) ||
	(vAxis == AlignDialog.DISTRIBUTE)) {
      for (int j = 0 ; j < shapes.length ; j++) {
	xOffset -= shapes[j].boundingBox.width;
	yOffset -= shapes[j].boundingBox.height;
      }

      xOffset /= (shapes.length - 1);
      yOffset /= (shapes.length - 1);
    }

    byte xDirection = 1;
    if (shapes[0].boundingBox.x > shapes[shapes.length - 1].boundingBox.x) {
      xDirection = -1;
    }

    byte yDirection = 1;
    if (shapes[0].boundingBox.y > shapes[shapes.length - 1].boundingBox.y) {
      yDirection = -1;
    }

    Rectangle previousShapeBBox = new Rectangle();
    previousShapeBBox.x = shapes[0].boundingBox.x - xDirection * xOffset;
    previousShapeBBox.y = shapes[0].boundingBox.y - yDirection * yOffset;

    for (int j = 0 ; j < shapes.length ; j++) {
      Rectangle shapeBBox = shapes[j].boundingBox;
			
      int dy = 0;
			
      if (hAxis == AlignDialog.TOP) {
	dy = bBox.y - shapeBBox.y;
      }
      else if (hAxis == AlignDialog.CENTER) {
	dy = (bBox.y + bBox.height / 2) - (shapeBBox.y + shapeBBox.height / 2);
      }
      else if (hAxis == AlignDialog.BOTTOM) {
	dy = (bBox.y + bBox.height) - (shapeBBox.y + shapeBBox.height);
      }
      else if (hAxis == AlignDialog.DISTRIBUTE) {
	dy = previousShapeBBox.y +
	     yDirection * (previousShapeBBox.height + yOffset) - shapeBBox.y;
      }
			
      int dx = 0;
			
      if (vAxis == AlignDialog.LEFT) {
	dx = bBox.x - shapeBBox.x;
      }
      else if (vAxis == AlignDialog.CENTER) {
	dx = (bBox.x + bBox.width / 2) - (shapeBBox.x + shapeBBox.width / 2);
      }
      else if (vAxis == AlignDialog.RIGHT) {
	dx = (bBox.x + bBox.width) - (shapeBBox.x + shapeBBox.width);
      }
      else if (vAxis == AlignDialog.DISTRIBUTE) {
	dx = previousShapeBBox.x +
	     xDirection * (previousShapeBBox.width + xOffset) - shapeBBox.x;
      }

      shapes[j].translate(dx, dy);

      previousShapeBBox = shapeBBox;
    }

    return true;
  }


  boolean rotate() {

    int i = 0;
    int nbProcessedShapes = 0;
				
    while (nbProcessedShapes < getNbSelectedShapes()) {
      Shape shape = (Shape) shapeList.elementAt(i);
      if (shape.selected) {
	shape.removeHandles();
	Point o1 = Geometry.createCenter(shape.boundingBox);
	shape.rotate(o1);
	shape.createHandles();

	// If the bounding box has an even dimension, o1 is an approximation.
	// Rotating around o1 might move the center of the bounding box.
	Point o2 = Geometry.createCenter(shape.boundingBox);
	if (!o2.equals(o1)) {
	  shape.translate(o1.x - o2.x, o1.y - o2.y);
	}
				
	nbProcessedShapes++;
      }
			
      i++;
    }
				
    return (nbProcessedShapes > 0);
		
  }


  boolean mirror(float xScale, float yScale) {

    int i = 0;
    int nbProcessedShapes = 0;
				
    while (nbProcessedShapes < getNbSelectedShapes()) {
      Shape shape = (Shape) shapeList.elementAt(i);
      if (shape.selected) {
	shape.removeHandles();
	Point o1 = Geometry.createCenter(shape.boundingBox);
	shape.scale(shape, o1, xScale, yScale);
	shape.createHandles();

	// If the bounding box has an even dimension, o1 is an approximation.
	// Scaling around o1 might move the center of the bounding box.
	Point o2 = Geometry.createCenter(shape.boundingBox);
	if (!o2.equals(o1)) {
	  shape.translate(o1.x - o2.x, o1.y - o2.y);
	}
				
	nbProcessedShapes++;
      }
			
      i++;
    }
				
    return (nbProcessedShapes > 0);
		
  }

	
  boolean group() {

    if (getNbSelectedShapes() < 2) {
      return false;
    }
		
    int i = 0;
    int nbProcessedShapes = 0;
    Shape[] shapes = new Shape[getNbSelectedShapes()];
		
    while (nbProcessedShapes < getNbSelectedShapes()) {
      Shape shape = (Shape) shapeList.elementAt(i);
      if (shape.selected == true) {
	shapeList.removeElementAt(i);
	shape.selected = false;
	shapes[nbProcessedShapes] = shape;
	nbProcessedShapes++;
      }
      else {
	i++;
      }
    }

    Group g = new Group(this, shapes);
    shapeList.insertElementAt(g, i);
    nothingSelected();
    g.createHandles();
		
    return true;
		
  }


  boolean ungroup() {
		
    int i = 0;
    int nbSelectedShapesFound = 0;
    int nbProcessedShapes = 0;
    int initialNbSelectedShapes = getNbSelectedShapes();
		
    while (nbSelectedShapesFound < initialNbSelectedShapes) {
      Shape shape = (Shape) shapeList.elementAt(i);
      if (shape.selected == true) {
	nbSelectedShapesFound++;
	if (shape instanceof Group) {
	  shapeList.removeElementAt(i);
	  shape.removeHandles();
	  Group g = (Group) shape;

	  for (int j = 0 ; j < g.shapes.length ; j++) {
	    Shape s = g.shapes[j];
	    shapeList.insertElementAt(s, i);
	    s.createHandles();
	    i++;
	  }

	  nbProcessedShapes++;
	}
	else {
	  i++;
	}
      }
      else {
	i++;
      }
    }
		
    return (nbProcessedShapes > 0);
  }


  public void paint(Graphics g) {

    // Shall we draw a grid?
    if (v.grid.getState()) {
      g.drawImage(grid.getImage(this), 0, 0, this);
    }
		
    // Draw existing shapes
    for (int i = 0 ; i < shapeList.size() ; i++) {
      ((Shape) shapeList.elementAt(i)).draw(g);
    }

    // Draw current shape:
    if ((anchor != null) && (currentPoint != null)) {
      if (areaSelection) {
	int x = Math.min(anchor.x, currentPoint.x);
	int y = Math.min(anchor.y, currentPoint.y);
	int width = Math.abs(anchor.x - currentPoint.x) + 1;
	int height = Math.abs(anchor.y - currentPoint.y) + 1;
	g.setColor(Color.black);
	g.setXORMode(Color.white);
	g.drawRect(x, y, width, height);
	g.setPaintMode();
      }
      else {
	if (currentPolygon.npoints != 0) {
	  g.setColor(v.tp.currentColor);
				
	  // Draw existing segments:
	  for (int i = 0 ; i < currentPolygon.npoints - 1 ; i++) {
	    g.drawLine(currentPolygon.xpoints[i], currentPolygon.ypoints[i],
		       currentPolygon.xpoints[i + 1],
		       currentPolygon.ypoints[i + 1]);
	  }
				
	  // Draw current segment:
	  Point p = new Point(currentPoint);

	  if (skl.isShiftDown()) {
	    p = Geometry.constrainPoint(p, anchor,
					new Rectangle(anchor,
						      new Dimension(1, 1)));
	  }
					
	  if (v.snapToGrid.getState() == true) {
	    p = grid.snap(p);
	  }
		
	  g.drawLine(anchor.x, anchor.y, p.x, p.y);
	}
	else {
	  Shape s = getCurrentShape(currentPoint.x, currentPoint.y);
	  if (s != null) {
	    s.draw(g);
	  }
	}
      }
    }

    // draw handles
    g.setXORMode(Color.white);
    for (int i = 0 ; i < handleList.size() ; i++) {
      ((Shape) handleList.elementAt(i)).draw(g);
    }
    g.setPaintMode();

  }

}
