import java.awt.*;
import java.awt.event.*;


class SheetMouseListener implements MouseListener {

  private Sheet s;
  Handle handleClicked;
  Shape initialShape;
  Shape extendedEditShape;


  public SheetMouseListener(Sheet s) {
    this.s = s;
  }


  public void mousePressed(MouseEvent e) {

    switch (s.v.tp.currentTool) {

    case ToolPanel.selection:
      Shape shape = s.findShape(e.getX(), e.getY());
      if (shape != null) {
	if (shape instanceof Handle) {
	  if (!(((Handle) shape).owner instanceof TextShape)) {
	    handleClicked = (Handle) shape;
	    initialShape = (Shape) handleClicked.owner.clone();
	    s.anchor = Geometry.createCenter(handleClicked.boundingBox);
	  }
	  break;
	}

	if (extendedEditShape != null) {
	  extendedEditShape = null;
	  s.handleList.removeAllElements();
	}
					
	if (e.getClickCount() == 1) {
	  if (e.isShiftDown()) {
	    if (shape.selected) {
	      shape.removeHandles();
	    }
	    else {
	      shape.createHandles();
	    }
	  }
	  else {
	    if (!shape.selected) {
	      s.unselectShapes();
	      shape.createHandles();
	    }
	  }
	}
	else if (e.getClickCount() == 2) {
	  if (shape instanceof TextShape) {
	    TextDialog td = new TextDialog((Frame) s.v, (TextShape) shape);
	    td.show();

	    TextShape ts = td.getTextShape();
	    if (ts != null) {  // shape must be replaced by ts:
	      shape.removeHandles();
	      s.shapeList.setElementAt(ts, s.shapeList.lastIndexOf(shape));
	      ts.createHandles();
	      s.repaint();
	    }
	  }
	  else if (shape instanceof PolygonShape) {
	    extendedEditShape = shape;
	    s.unselectShapes();
	    PolygonShape polygon = (PolygonShape) shape;
	    polygon.createExtendedHandles();
	  }
	}
      }
      else {
	if (extendedEditShape != null) {
	  extendedEditShape = null;
	  s.handleList.removeAllElements();
	}
					
	s.areaSelection =  true;
	s.anchor = e.getPoint();  // starting point of selection rectangle
      }

      if ((shape != null) && (shape.selected)) {
	s.anchor = e.getPoint();  // starting point of translation

	if (s.v.snapToGrid.getState() == true) {
	  s.anchor = s.grid.snap(s.anchor);
	}
      }

      s.repaint();
      break;

    case ToolPanel.line:
    case ToolPanel.rectangle:
    case ToolPanel.ellipse:
      s.anchor = e.getPoint();

      if (s.v.snapToGrid.getState() == true) {
	s.anchor = s.grid.snap(s.anchor);
      }
      break;			

    case ToolPanel.polygon:
      if (e.getClickCount() == 1) {
	Point p = e.getPoint();

	if (e.isShiftDown() && (s.currentPolygon.npoints != 0)) {
	  Point lastPoint =
	    new Point(s.currentPolygon.xpoints[s.currentPolygon.npoints - 1],
		      s.currentPolygon.ypoints[s.currentPolygon.npoints - 1]);
	  p = Geometry.constrainPoint(p, lastPoint,
				      new Rectangle(lastPoint,
						    new Dimension(1, 1)));
	}
					
	if (s.v.snapToGrid.getState() == true) {
	  p = s.grid.snap(p);
	}
					
	s.currentPolygon.addPoint(p.x, p.y);
	s.anchor = p;
      }
      else {
	s.addShape(e.getX(), e.getY());
	s.currentPolygon = new Polygon();
	s.anchor = null;
      }
      break;

    case ToolPanel.text:
      s.anchor = e.getPoint();

      if (s.v.snapToGrid.getState() == true) {
	s.anchor = s.grid.snap(s.anchor);
      }
				
      TextDialog td = new TextDialog((Frame) s.v, null);
      td.show();

      TextShape ts = td.getTextShape();
      if (ts != null) {
	s.shapeList.addElement(ts);
	s.repaint();
      }
      s.anchor = null;
      break;
    }
    
  }


  public void mouseReleased(MouseEvent e) {

    switch (s.v.tp.currentTool) {

    case ToolPanel.line:
    case ToolPanel.rectangle:
    case ToolPanel.ellipse:
      if (s.anchor != null) {
	s.addShape(e.getX(), e.getY());
      }
      s.anchor = null;
      break;

    case ToolPanel.selection:
      if (s.areaSelection) {
	if ((s.currentPoint != null) && (!s.currentPoint.equals(s.anchor))) {
	  Rectangle selectRectangle = new Rectangle(s.anchor);
	  selectRectangle.add(s.currentPoint);
	  s.areaSelection(selectRectangle, e.isShiftDown());
	}
	else {
	  if (!e.isShiftDown()) {
	    s.unselectShapes();
	  }
	}
					
	s.areaSelection = false;
	s.currentPoint = null;
      }
  			
      s.repaint();
      s.anchor = null;
      handleClicked = null;
      initialShape = null;
      break;
    }

    s.smml.previousPosition = null;
    
  }


  public void mouseEntered(MouseEvent e) {
    
    s.requestFocus();
    
  }


  public void mouseClicked(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}

}
