import java.awt.*;
import java.awt.event.*;


class SheetMouseMotionListener implements MouseMotionListener {

  private Sheet s;
  Point mousePosition;
  Point previousPosition;


  public SheetMouseMotionListener(Sheet s) {
    this.s = s;
  }


  public void mouseDragged(MouseEvent e) {
		
    s.requestFocus();

    mousePosition = e.getPoint();
    Point p = new Point(mousePosition);

    switch (s.v.tp.currentTool) {

    case ToolPanel.line:
    case ToolPanel.rectangle:
    case ToolPanel.ellipse:
      s.currentPoint = e.getPoint();
      s.repaint();
      break;

    case ToolPanel.selection:
      if (s.anchor != null) {
	if (s.areaSelection) {
	  s.currentPoint = e.getPoint();
	  s.repaint();
	}
	else {  // translation
						
	  if (previousPosition == null) {
	    previousPosition = s.anchor;
	  }

	  p = translation(p);
	}
      }
      break;
    }

    previousPosition = p;
    
  }


  public void mouseMoved(MouseEvent e) {

    s.requestFocus();

    mousePosition = e.getPoint();
		
    if (s.currentPolygon.npoints != 0) {
      s.currentPoint = e.getPoint();
      s.repaint();
    }
    
  }


  Point translation(Point p) {
		
    if (s.sml.handleClicked != null) {
      Shape shape = s.sml.handleClicked.owner;

      if (s.sml.extendedEditShape != null) {
	if (s.sml.extendedEditShape instanceof PolygonShape) {
	  if (s.skl.isShiftDown()) {
	    p = Geometry.constrainPoint(p, s.anchor,
					new Rectangle(s.anchor,
						      new Dimension(1, 1)));
	  }

	  if (s.v.snapToGrid.getState() == true) {
	    p = s.grid.snap(p);
	  }
					
	  PolygonShape polygon = (PolygonShape) s.sml.extendedEditShape;
	  polygon.extendedEdit(previousPosition, p);

	  int dx = p.x - previousPosition.x;
	  int dy = p.y - previousPosition.y;
	  s.sml.handleClicked.translate(dx, dy);
	}
      }
      else {
	if (s.skl.isShiftDown()) {
	  p = Geometry.constrainPoint(p, s.anchor,
				      s.sml.initialShape.boundingBox);
	}

	if (s.v.snapToGrid.getState() == true) {
	  p = s.grid.snap(p);
	}
				
	shape.reshape(s.sml.initialShape, s.anchor, p);
	shape.removeHandles();
	shape.createHandles();
      }
    }
    else {
      if (s.skl.isShiftDown()) {
	p = Geometry.constrainPoint(p, s.anchor,
				    new Rectangle(s.anchor,
						  new Dimension(1, 1)));
      }

      if (s.v.snapToGrid.getState() == true) {
	p = s.grid.snap(p);
      }
			
      int dx = p.x - previousPosition.x;
      int dy = p.y - previousPosition.y;
      s.translateShapes(dx, dy);
    }
		
    s.repaint();

    return p;
		
  }

}
