import java.awt.*;
import java.awt.event.*;


class TextDialog extends Dialog implements ActionListener {

  TextField text;
  Choice fontChoice;
  Checkbox bold;
  Checkbox italic;
  TextField fontSize;
  OkCancelPanel ocp;

  private TextShape textShape;
	

  public TextDialog(Frame f, TextShape textShape) {
		
    super(f, "Text Object", true);
    setLocation(60, 60);
		
    addWindowListener((WindowListener) new WindowWindowListener(this));

    this.textShape = textShape;

    Panel t = new Panel(new BorderLayout());
    t.add("North", new Label("Text:"));
    String s = "";
    if (textShape != null) {
      s = textShape.text;
    }
    text = new TextField(s);
    t.add("South", text);

    fontChoice = new Choice();
    String[] fonts = Toolkit.getDefaultToolkit().getFontList();
    for (int i = 0 ; i < fonts.length ; i++) {
      fontChoice.add(fonts[i]);
    }
    String fontName = "Serif";
    if (textShape != null) {
      fontName = textShape.font.getName();
    }
    fontChoice.select(fontName);

    Panel northFontNamePanel = new Panel(new BorderLayout());
    northFontNamePanel.add("North", new Label("Font name:"));
    northFontNamePanel.add("South", fontChoice);
		
    Panel fontNamePanel = new Panel(new BorderLayout());
    fontNamePanel.add("North", northFontNamePanel);

    GridBagLayout gridbag = new GridBagLayout();
    Panel fontStylePanel = new Panel(gridbag);
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.anchor = GridBagConstraints.WEST;

    Label fontStyle = new Label("Font style:");
    gridbag.setConstraints(fontStyle, gbc);
    fontStylePanel.add(fontStyle);
		
    bold = new Checkbox(" bold");
    italic = new Checkbox(" italic");
    if (textShape != null) {
      if (textShape.font.isBold()) {
	bold.setState(true);
      }
		
      if (textShape.font.isItalic()) {
	italic.setState(true);
      }
    }

    gridbag.setConstraints(bold, gbc);
    fontStylePanel.add(bold);

    gridbag.setConstraints(italic, gbc);
    fontStylePanel.add(italic);

    Panel northFontSizePanel = new Panel(new BorderLayout());
    northFontSizePanel.add("North", new Label("Font size:"));
    int size = 12;
    if (textShape != null) {
      size = textShape.font.getSize();
    }
    fontSize = new TextField(String.valueOf(size));
    northFontSizePanel.add("South", fontSize);
		
    Panel fontSizePanel = new Panel(new BorderLayout());
    fontSizePanel.add("North", northFontSizePanel);

    Panel fontAttributesPanel = new InsetsPanel(new Insets(10, 0, 10, 0),
						new GridLayout(1, 3, 40, 10));
    fontAttributesPanel.add(fontNamePanel);
    fontAttributesPanel.add(fontStylePanel);
    fontAttributesPanel.add(fontSizePanel);

    ocp = new OkCancelPanel();
    ocp.ok.addActionListener(this);
    ocp.cancel.addActionListener(this);

    InsetsPanel mainPanel = new InsetsPanel(new Insets(10, 10, 10, 10),
					    new BorderLayout());
    mainPanel.add("North", t);
    mainPanel.add("Center", fontAttributesPanel);
    mainPanel.add("South", ocp);

    add(mainPanel);

    pack();
  }

	
  TextShape getTextShape() {
    return textShape;
  }

	
  public void actionPerformed(ActionEvent e) {

    if (e.getSource() == ocp.ok) {
      String t = text.getText();
      if (t.equals("")) {
	textShape = null;
      }
      else {
	int style = Font.PLAIN;
	if (bold.getState()) {
	  style += Font.BOLD;
	}
	if (italic.getState()) {
	  style += Font.ITALIC;
	}

	int size = Integer.parseInt(fontSize.getText());
			
	Font font = new Font(fontChoice.getSelectedItem(), style, size);
	Victor v = (Victor) getParent();
				
	Point p;
	Color c;
	if (textShape != null) {  // Modifying an existing TextShape.
	  p = textShape.p;
	  c = textShape.color;
	}
	else {  // Creating a new TextShape.
	  p = v.s.anchor;
	  c = v.tp.currentColor;
	}
				
	textShape = new TextShape(v.s, p, t, font, c);
      }
    }
    else if (e.getSource() == ocp.cancel) {
      textShape = null;
    }

    setVisible(false);
    dispose();
  }

}
