import java.awt.*;
import java.io.*;


class TextShape extends Shape {

	Point p;
	String text;
	Font font;


	public TextShape(Sheet s, Point p, String text, Font font, Color c) {
		this.s = s;
		color = c;
				
		this.p = p;
		this.text = text;
		this.font = font;

		createBoundingBox();
	}


	public TextShape(Sheet s, BufferedReader in) {
		this.s = s;
		color = InOut.readColor(in);
		p = InOut.readPoint(in);
		
		try {
			text = in.readLine();
		}
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }

    font = InOut.readFont(in);

		createBoundingBox();
		
		try {
			in.readLine();	// skip blank line
		}
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }
	}


	void createBoundingBox() {
		boundingBox = new Rectangle(p);
		FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(font);
		boundingBox.width = fm.stringWidth(text);
		boundingBox.height = fm.getHeight();
	}

	
	protected Object clone() {
		return new TextShape(s, new Point(p), text, font, color);
	}

	
	public String toString() {
		String s;

		s = "Text\n";
		s = s + InOut.writeColor(color);
		s = s + InOut.writePoint(p);
		s = s + text + "\n";
		s = s + InOut.writeFont(font) + "\n";
		
		return s;
	}

	
	void draw(Graphics g) {
		g.setColor(color);
		g.setFont(font);
		FontMetrics fm = g.getFontMetrics();
		g.drawString(text, p.x, p.y + fm.getMaxAscent());
	}


	boolean contains(int x, int y) {
		return boundingBox.contains(x, y);
	}


	void translate(int dx, int dy) {
		
		boundingBox.translate(dx, dy);
		
		p.translate(dx, dy);

		if (selected) {
			removeHandles();
			createHandles();
		}
		
	}


	void rotate(Point c) {
		
	}


	void scale(Shape initialShape, Point o, float xScale, float yScale) {
		
		Point p1 = Geometry.createCenter(boundingBox);
		Point p2 = Geometry.scale(Geometry.createCenter(initialShape.boundingBox),
															o, xScale, yScale);
		translate(p2.x - p1.x, p2.y - p1.y);
		
		createBoundingBox();
		
	}

	
	void reshape(Shape initialShape, Point start, Point end) {
	}

}

